float4 vecSunColor;
float4 vecLightPos[8]; //light position
float4 vecLightColor[8]; //light position                          // power
vector vecAmbient;
vector vecDiffuse;
vector vecSpecular;
vector vecEmissive;
float4 vecLight;
// transformations
float4x4 matWorld;
float4x4 matView;
float4x4 matViewProj;
float4x4 matProj;
float4x4 matWorldView;
float4x4 matWorldViewProj;
float4x4 matWorldInv;
float4x4 matViewInv;
float4 vecViewPos;
float4 vecSunDir;
float4 vecFog;
float4 vecTime;

texture entSkin1;


sampler ColorSampler =sampler_state
{
	Texture=<entSkin1>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};


float intersectRaySphere(float3 rO, float3 rV, float3 sO, float sR) {
	
   
	float3 Q = sO-rO;
   
   
	float c = length(Q);
   
	float v = dot(Q,rV);
   
	float d = sR*sR - (c*c - v*v);

   // If there was no intersection, return -1
   
	//	if (d < 0.0) return (-1.0f);

   // Return the distance to the [first] intersecting point
   
	return (v - sqrt(d));

}




// vertex shader output structure
struct VS_OUTPUT
{
	float4 Pos  : POSITION;
	float2 Tex : TEXCOORD0; 
	float  Fog:  FOG;
	float3 Sun : TEXCOORD1;
	float3 ViewX : TEXCOORD2;
	float layer:TEXCOORD3;
	float3 SunColor: TEXCOORD4;
	float3 vPos:TEXCOORD5;
};



VS_OUTPUT VS(
float4 InPos  : POSITION,   // Vertex position in model space
float3 InNormal : NORMAL,   // Vertex normal in model space
float2 texcoord1 : TEXCOORD1,
float2 texcoord2 : TEXCOORD0,
float3 inTangent  : TEXCOORD2)       
{
	VS_OUTPUT Out = (VS_OUTPUT)0;
	InPos.xyz*=0.5;
	InPos.y=0;
	float4 Pos = mul(InPos, (float4x4)matWorld);
	float height=Pos.y*0;
	float radius=length(Pos.xz);
	float tilt=acos(radius/8000);//
	tilt=radians(radius/7000*90);
	float pan=atan2(Pos.x, Pos.z);
	radius=8500;
	Pos.x=radius*sin(pan)*sin(tilt);
	Pos.z=radius*cos(pan)*sin(tilt);
	Pos.y=radius*cos(tilt)-7000;
	float3 ortho=float3(sin(pan)*sin(tilt), cos(tilt), cos(pan)*sin(tilt));
	Pos.xyz+=height*ortho;
	float atmo_radius=10000;//*0.125;
	float p_rad=7000;//*0.125;
	float3 vRay=Pos.xyz-vecViewPos.xyz;
	float far=length(vRay);
	float near=intersectRaySphere(vecViewPos.xyz, normalize(vRay),float3(0,-7000,0),  radius);
	float thickness = (far-max(near,0));///(radius*2);
	Out.layer=0;
	Out.SunColor=0;
	for(int i=1; i<10; i++)
	{
		float3 p1=Pos.xyz+thickness*(i/10.0)*normalize(-vRay);
		float near=-intersectRaySphere(p1.xyz, normalize(-vecSunDir),float3(0,-7000,0),  radius)/(radius*2);
		float moist=1-(length(float3(p1.x, p1.y+7000, p1.z))-p_rad)/(radius-p_rad);

		Out.layer+=near;
		float3 light=moist*thickness/(radius*2)*near*float3(0.1,0.5,1.0)*5;
		light+=moist*pow(max(dot(normalize(vRay), -vecSunDir),0),30)*0.3;
		light-=moist*(thickness*(1-i/10.0)/(radius*2))*float3(0.01,0.4,1.0)*6;
		Out.SunColor+=max(light,0);
	}
	Out.layer=thickness/(radius*2);
	
	Out.Pos  = mul(mul(Pos, matView), matProj); 
	Out.layer=-Pos.y/300;
	Out.Tex=texcoord1*1.5;
	Out.Fog = 1 - (distance(Pos, vecViewPos) - vecFog.x) * vecFog.z;
	float3x3 matTangent;
	float3 ortho2=float3(sin(pan)*sin(tilt), cos(tilt), cos(pan)*sin(tilt));
	InNormal=normalize(ortho2);
	inTangent = normalize(cross(InNormal, float3(0,0,1)));
	matTangent[0] =inTangent;// mul(inTangent, matWorld);
	matTangent[1] = cross(inTangent.xyz, InNormal.xyz);//, matWorld);
	matTangent[2] = InNormal;//mul(InNormal, matWorld);
	//   Out.normal=InNormal;
	Out.Sun.xyz = mul(matTangent, -vecSunDir);	
	float3 ViewerX = Pos - vecViewPos;	
	Out.ViewX =Pos;
	return Out;
}




float cloudDensity(float2 coord)
{	
	float4 nmap=tex2D(ColorSampler, coord.xy*10+0.125*float2(vecTime.w*0.005,vecTime.w*0.005));
	nmap.a+=tex2D(ColorSampler, coord.xy*6+0.25*float2(vecTime.w*0.001,0)).a;
	nmap.a+=tex2D(ColorSampler, coord.xy*5+0.25*float2(vecTime.w*0.005,vecTime.w*0.005)).a;
	nmap.a+=tex2D(ColorSampler, coord.xy*8+0.25*float2(vecTime.w*0.002,vecTime.w*0.002)).a;
	nmap.a/=4;
	return nmap.a;
}


float4 PS ( VS_OUTPUT In) : COLOR
{
	float4 OutColor;
	
	float4 color = tex2D(ColorSampler, In.Tex);
	float4 nmap=tex2D(ColorSampler, In.Tex.xy*10);
	float3 ResultNormalmap = 2*(nmap.xyz)-1;
	nmap.a=cloudDensity(In.Tex.xy);
	float2 vPixelSize=float2(1.3/512.0,1.3/512.0);
	
	//calculate normalmap
	float dist=0.9/512.0;
	vPixelSize=float2(0.9/512.0,-0.9/512.0);
	float hR=cloudDensity(float2(In.Tex.x+vPixelSize.x,In.Tex.y)).x;
	float hRT=cloudDensity(float2(In.Tex.x+vPixelSize.x,In.Tex.y+vPixelSize.y)).x;
	float hRB=cloudDensity(float2(In.Tex.x+vPixelSize.x,In.Tex.y-vPixelSize.y)).x;
	float hL=cloudDensity(float2(In.Tex.x-vPixelSize.x,In.Tex.y)).x;
	float hLT=cloudDensity(float2(In.Tex.x-vPixelSize.x,In.Tex.y+vPixelSize.y)).x;
	float hLB=cloudDensity(float2(In.Tex.x-vPixelSize.x,In.Tex.y-vPixelSize.y)).x;
	float hT=cloudDensity(float2(In.Tex.x,In.Tex.y+vPixelSize.y)).x;
	float hB=cloudDensity(float2(In.Tex.x,In.Tex.y-vPixelSize.y)).x;
	float ddx=(2*hR+hRT+hRB)/4.0-(2*hL+hLT+hLB)/4.0;
	float ddy=(2*hT+hRT+hLT)/4.0-(2*hB+hRB+hLB)/4.0;
	float3 vec1=float3(1,0,ddx);
	float3 vec2=float3(0,1,ddy);
	
	ResultNormalmap=cross(vec1,vec2)*5;
	ResultNormalmap.z=0.5;
	ResultNormalmap=normalize(ResultNormalmap);
	
	float3 diffColor = 0.0;//vecAmbient.xyz;
	float3 specColor = 0.0f;

	float3 sun=normalize(In.Sun);
	float3 viewx=normalize(In.ViewX);
	float diffLight =max(dot(sun, ResultNormalmap),0);
	float3 sunColor = lerp(float3(1,0.3,0.1), float3(0.95,0.98,1), pow(vecSunDir.y,0.4));
	diffColor += diffLight*sunColor;//    

	float fogStart=5;
	float fogEnd=5000;
	float fog= saturate((distance(In.ViewX.xyz, vecViewPos.xyz) - fogStart) /(fogEnd-fogStart));//-100)*(1/1000);
	float3 darkColor=lerp(sunColor*0.75, float3(0.4,0.6,0.8), fog);
	
	OutColor.rgb =diffLight*1.0;//lerp(darkColor,sunColor*1, min(diffLight*0.8+0.2,0.9));
	OutColor.rgb+=In.SunColor*1;
	OutColor.rgb=1-exp(-OutColor.rgb*0.75);
	OutColor.a=1;
	nmap.a*=1.45;
	OutColor.a*=pow(nmap.a,15);
	return OutColor;
}


VS_OUTPUT VS2(
float4 InPos  : POSITION, 
float3 InNormal : NORMAL,
float2 texcoord1 : TEXCOORD1,
float2 texcoord2 : TEXCOORD0,
float3 inTangent  : TEXCOORD2)       
{
	VS_OUTPUT Out = (VS_OUTPUT)0;
	InPos.xyz*=0.5;
	InPos.y=0;
	float4 Pos = mul(InPos, (float4x4)matWorld);   
	float height=Pos.y*0;
	float radius=length(Pos.xz);
	float tilt=acos(radius/8000);//
	tilt=radians(radius/7000*90);
	float pan=atan2(Pos.x, Pos.z);
	radius=10000;
	Pos.x=radius*sin(pan)*sin(tilt);
	Pos.z=radius*cos(pan)*sin(tilt);
	Pos.y=radius*cos(tilt)-7000;
	float3 ortho=float3(sin(pan)*sin(tilt), cos(tilt), cos(pan)*sin(tilt));
	Pos.xyz+=height*ortho;
	float p_rad=7000;//*0.125;
	float3 vRay=Pos.xyz-vecViewPos.xyz;
	float far=length(vRay);
	float near=intersectRaySphere(vecViewPos.xyz, normalize(vRay),float3(0,-7000,0),  radius);
	float thickness = (far-max(near,0));///(radius*2);
	Out.layer=0;
	Out.Sun=0;
	for(int i=1; i<10; i++)
	{
		float3 p1=Pos.xyz+thickness*(i/10.0)*normalize(-vRay);
		float near=-intersectRaySphere(p1.xyz, normalize(-vecSunDir),float3(0,-7000,0),  radius)/(radius*2);
		float moist=1-(length(float3(p1.x, p1.y+7000, p1.z))-p_rad)/(radius-p_rad);
		float3 light=moist*thickness/(radius*2)*near*float3(0.1,0.5,1.0)*5;
		light+=moist*pow(max(dot(normalize(vRay), -vecSunDir),0),20)*0.4;
		light-=moist*(thickness*(1-i/10.0)/(radius*2))*float3(0.01,0.4,1.0)*6;
		Out.Sun+=max(light,0);
	}
	Out.Sun=1-exp(-Out.Sun*0.75);
	Out.layer=thickness/(radius*2);
	
	
	Out.Pos  = mul(mul(Pos, matView), matProj); 
	Out.Tex=texcoord1*1.5;
	Out.Fog = 1 - (distance(Pos, vecViewPos) - vecFog.x) * vecFog.z;


	float3x3 matTangent;
	matTangent[0] = mul(inTangent, matWorld);
	matTangent[1] = mul(cross(inTangent.xyz, InNormal.xyz), matWorld);
	matTangent[2] = mul(InNormal, matWorld);
	float3 ViewerX = Pos - vecViewPos;	
	Out.ViewX =Pos; 
	return Out;
}

float4 PS2 ( VS_OUTPUT In) : COLOR
{
	float4 OutColor=1;	
	OutColor.rgb =In.Sun;
	OutColor.a=1;
	return OutColor;
}
technique normalmap
{
	pass blue
	{
		CullMode=cw;
		AlphaBlendEnable=true;
		ZEnable=True;
		ZWriteEnable=false;
		//	AlphaTestEnable=true;
		//fillmode=2;
		BlendOp=add;
		SrcBlend=one;
		DestBlend=one;
		VertexShader = compile vs_3_0 VS2();
		PixelShader  = compile ps_3_0 PS2();
	}
	pass P0
	{
		CullMode=cw;
		AlphaBlendEnable=true;
		ZEnable=True;
		ZWriteEnable=false;
		AlphaTestEnable=true;
		SrcBlend=srcalpha;
		DestBlend=invsrcalpha;
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
	}
	pass P0
	{
		CullMode=ccw;
		AlphaBlendEnable=true;
		ZEnable=True;
		ZWriteEnable=false;
		AlphaTestEnable=true;
		SrcBlend=srcalpha;
		DestBlend=invsrcalpha;
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
	}
	//    pass blue
	//    {
		//    	CullMode=ccw;
		//	AlphaBlendEnable=true;
		//	ZEnable=True;
		//	ZWriteEnable=false;
		////	AlphaTestEnable=true;
		//fillmode=2;
		//        VertexShader = compile vs_3_0 VS2();
		//        PixelShader  = compile ps_3_0 PS2();
	//    }
}
