float4 vecSunColor;
float4 vecLightPos[8]; //light position
float4 vecLightColor[8]; //light position                          // power
vector vecAmbient;
vector vecDiffuse;
vector vecSpecular;
vector vecEmissive;
float4 vecLight;
// transformations
float4x4 matWorld;
float4x4 matView;
float4x4 matProj;
float4x4 matWorldView;
float4x4 matWorldViewProj;
float4x4 matWorldInv;
float4x4 matViewInv;
float4 vecViewPos;
float4 vecSunDir;
float4 vecFog;
float4 vecTime;

texture entSkin1;
texture entSkin2;
texture entSkin3;
texture mtlSkin1;


sampler ColorSampler =sampler_state
{
	Texture=<entSkin1>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};
sampler NormalSampler =sampler_state
{
	Texture=<entSkin2>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};
sampler WaterSampler =sampler_state
{
	Texture=<entSkin3>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};
sampler DetailSampler =sampler_state
{
	Texture=<mtlSkin1>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};






// vertex shader output structure
struct VS_OUTPUT
{
	float4 Pos  : POSITION;
	float2 Tex : TEXCOORD0; 
	float  Fog:  FOG;
	float3 Sun : TEXCOORD1;
	float3 ViewX : TEXCOORD2;
	float layer:TEXCOORD3;
	float3 SunColor:TEXCOORD4;
	float3 vPos:TEXCOORD5;
};



float intersectRaySphere(float3 rO, float3 rV, float3 sO, float sR) {
	
   
	float3 Q = sO-rO;
   
   
	float c = length(Q);
   
	float v = dot(Q,rV);
   
	float d = sR*sR - (c*c - v*v);

   // If there was no intersection, return -1
   
	//	if (d < 0.0) return (-1.0f);

   // Return the distance to the [first] intersecting point
   
	return (v - sqrt(d));

}

VS_OUTPUT VS(
float4 InPos  : POSITION,   // Vertex position in model space
float3 InNormal : NORMAL,   // Vertex normal in model space
float2 texcoord1 : TEXCOORD1,
float2 texcoord2 : TEXCOORD0,
float3 inTangent  : TEXCOORD2)       
{
	VS_OUTPUT Out = (VS_OUTPUT)0;

	
	float4 Pos = mul(InPos, (float4x4)matWorld);        
	float height=Pos.y*0.95;
	float radius=length(Pos.xz);
	float tilt=3.14-2*acos(radius/8000);//
	tilt=radians(radius/7000*70);
	float pan=atan2(Pos.x, Pos.z);
	radius=7000;
	Pos.x=radius*sin(pan)*sin(tilt);
	Pos.z=radius*cos(pan)*sin(tilt);
	Pos.y=radius*cos(tilt)-7000;
	float3 ortho=float3(sin(pan)*sin(tilt), cos(tilt), cos(pan)*sin(tilt));
	Pos.xyz+=height*ortho;
	float atmo_radius=10000;//*0.125;
	float3 vRay=Pos.xyz-vecViewPos.xyz;
	float far=length(vRay);
	float near=intersectRaySphere(vecViewPos.xyz, normalize(vRay),float3(0,-7000,0),  atmo_radius);
	float thickness = (far-max(near,0));///(radius*2);
	Out.layer=0;
	Out.SunColor=0;    
	for(int i=1; i<10; i++)
	{
		float3 p1=Pos.xyz+thickness*(i/10.0)*normalize(-vRay);
		float near=1+intersectRaySphere(p1.xyz, normalize(vecSunDir),float3(0,-7000,0),  atmo_radius)/(atmo_radius*2);
		float moist=pow(1-(length(float3(p1.x, p1.y+7000, p1.z))-radius)/(atmo_radius-radius),3);
		moist=1-(length(float3(p1.x, p1.y+7000, p1.z))-radius)/(atmo_radius-radius);
		moist=pow(moist,1.5)*1.2;
		float3 light=moist*thickness/(atmo_radius*2)*near*float3(0.1,0.5,1.0)*5;
		light-=moist*(thickness*(1-i/10.0)/(atmo_radius*2))*float3(0.01,0.4,1.0)*6;
		Out.SunColor+=max(light,0);
	}
	Out.Pos  = mul(mul(Pos, matView), matProj);    
	Out.layer=pow(thickness/(atmo_radius*2)*2.5,0.9)*1;
	Out.Tex=texcoord2;
	Out.Fog = 1 - (distance(Pos, vecViewPos) - vecFog.x) * vecFog.z;


	float3x3 matTangent;
	float3 ortho2=float3(sin(pan)*sin(tilt), cos(tilt), cos(pan)*sin(tilt));
	InNormal=normalize(ortho2);
	inTangent = normalize(cross(InNormal, float3(0,0,1)));
	matTangent[0] = mul(inTangent, matWorld);
	matTangent[1] = mul(cross(inTangent.xyz, InNormal.xyz), matWorld);
	matTangent[2] = mul(InNormal, matWorld);
	Out.Sun.xyz = mul(matTangent, -vecSunDir);	
	float3 ViewerX = Pos - vecViewPos;	
	Out.ViewX =Pos;    
	return Out;
}




float4 PS ( VS_OUTPUT In) : COLOR
{
	float4 OutColor;
	
	float4 color = tex2D(ColorSampler, In.Tex);
	float4 nmap=tex2D(NormalSampler, In.Tex.xy);
	float4 water=tex2D(WaterSampler, float2(In.Tex.x, 1-In.Tex.y));
	float4 detail = tex2D(DetailSampler, In.Tex*30);
	detail.r=detail.r*0.75+0.25;
	float3 ResultNormalmap = 2*(nmap.xyz)-1;    
	float3 diffColor = 0.0;
	float3 specColor = 0.0f;
	float3 sun=normalize(In.Sun);
	float3 viewx=normalize(In.ViewX);
	float diffLight =max(dot(sun, ResultNormalmap),0);
	float3 sunColor = In.SunColor;//lerp(float3(1,0.3,0.1), float3(0.95,0.98,1), pow(vecSunDir.y,0.4));
	diffColor += diffLight;//*sunColor;//    

	OutColor.rgb =color.rgb*(0.0+diffColor)*detail.r;
	OutColor.rgb +=In.SunColor;
	OutColor.rgb=1-exp(-OutColor.rgb*0.75);
	OutColor.a=1;
	return OutColor;
}



technique normalmap
{
	pass P0
	{
		AlphaBlendEnable=false;
		ZEnable=True;
		ZWriteEnable=true;
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
	}
}
