float4x4 matView;
float4x4 matProj;
float4x4 matWorld;
float4x4 matWorldViewProj;
float4x4 matLight;
float4x4 matShadow;
float4 vecSunDir;
float4 vecViewPos;
float4 vecShadPos;
float4 vecAtmoColor;
float4 vecSunColor;

texture entSkin1;
texture entSkin2;
texture entSkin3;

sampler ColorSampler =sampler_state
{
	Texture=<entSkin1>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	
	AddressU  = wrap;
	AddressV  = wrap;
};
sampler LightSampler =sampler_state
{
	Texture=<entSkin2>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	
	AddressU  = clamp;
	AddressV  = clamp;
};
sampler ShadowSampler =sampler_state
{
	Texture=<entSkin3>;
	
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	
	AddressU  = clamp;
	AddressV  = clamp;
};

struct VS_OUTPUT
{
	float4 Pos: POSITION;
	float2 Tex: TEXCOORD0;
	float4 SpotPos:TEXCOORD1;
	float4 Diffuse:COLOR0;
	float4 Specular:COLOR1;
	float2  AO	:TEXCOORD2;
	float3 WPos:TEXCOORD3;
	float4 Tex2:	TEXCOORD4;
	float3 BlendWeights:TEXCOORD5;
	float4 ShadowPos:TEXCOORD6;
	float4 SunColor:TEXCOORD7;
};


VS_OUTPUT VS(float4 inPos:POSITION, float3 inNormal:NORMAL, float2 Tex:TEXCOORD0)
{
	VS_OUTPUT Out=(VS_OUTPUT)0;
	Out.Pos=mul(inPos,matWorldViewProj);
	inNormal=normalize(mul(normalize(inNormal),matWorld));
	float4 PosW = mul(inPos, (float4x4)matWorld);
	Out.SpotPos=mul(PosW,matLight);
	Out.ShadowPos=mul(PosW,matShadow);
	Out.WPos=PosW*10;	
	
	Out.Diffuse=max(0,dot(inNormal,vecSunDir.xyz));
	float3 ViewerX = Out.WPos.xyz - vecViewPos.xyz;	
	float specular=(Out.Diffuse>0)?saturate(pow(saturate(dot(inNormal,normalize(-ViewerX + normalize(-vecSunDir)))),5)):0;
	Out.Specular=specular;
	Out.AO.x=Tex.x;
	Out.AO.y=Tex.y/100;//inNormal.x;//Tex.x;
	float3 blend_weights = abs( normalize(inNormal.xyz) );
	blend_weights = (blend_weights - 0.2) * 7;
	blend_weights = max(blend_weights, 0);
	blend_weights /= (blend_weights.x + blend_weights.y +	blend_weights.z ).xxx;
	Out.BlendWeights=blend_weights;
	Out.Tex=Out.WPos.yz*0.005;
	Out.Tex2.zw=Out.WPos.xy*0.005;
	Out.Tex2.xy=Out.WPos.xz*0.005;
	float3 objLightPos  =vecViewPos.xyz;
	float3 objLightDir  = objLightPos-PosW;
	float dis=length(objLightDir);
	float Atten =0;
	float dif=length(objLightDir.xyz)/(100);
	Atten =1/( 0.0 +1.0 * dif + 1 * dif*dif);
	float3 objLightDirN = normalize(objLightDir.xyz);
	float  diffLight    =max(dot(inNormal,objLightDirN.xyz),0);//, InNormal.xyz);
	Out.Diffuse.a  =Atten*diffLight;	
	
	float4 Pos = mul(inPos, (float4x4)matWorld);
	Out.SunColor.rgb=vecAtmoColor.rgb;
	Out.Diffuse.rgb*=vecSunColor.rgb;
	return Out;
}


float4 PS(VS_OUTPUT In):COLOR0
{
	float4 color;
	// Use warped world-space coordinate to generate a marble texture.
	float4 color1=tex2D(ColorSampler,In.Tex);
	float4 color2=tex2D(ColorSampler,In.Tex2.xy);
	float4 color3=tex2D(ColorSampler,In.Tex2.zw);
	color=color1*In.BlendWeights.x+color2*In.BlendWeights.y+color3*In.BlendWeights.z;
	float3 coloring=lerp(float3(0.5,0.7,1.0)*0.9,float3(0.4,0.7,0.2),saturate((In.WPos.y-25220)/20));
	coloring=lerp(coloring, float3(0.5,0.7,1.0), 0);
	color.rgb*=coloring;
	float2 spotCoord=In.SpotPos.xy/In.SpotPos.w;
	float4 spotColor=tex2D(LightSampler,spotCoord)*In.Diffuse.a;
	float ao=saturate(1.5*In.AO.x);	
	float3 sunColor = lerp(float3(1,0.3,0.1), float3(0.95,0.98,1), pow(vecSunDir.y,0.4));
	color.rgb*=(In.Diffuse.rgb*sunColor*0.8+0.01)*2*ao+spotColor*0.5;	
	color.rgb=color.rgb*2+In.SunColor.rgb;
	color.rgb=1-exp(-color.rgb*0.75);
	color.a=1;
	return color;	
}




technique renderTerrain
{

	pass P0
	{
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
		ZEnable = true;
		ZWriteEnable=true;
		AlphaBlendEnable = false;
		CullMode=ccw;
	}	
	
}